@@ -1,6 +1,6 @@
 /*
  *  Breeze  --  An application launcher with command-line style
- *  Copyright (C) 2005, 2006 Hironao Komatsu
+ *  Copyright (C) 2005, 2006, 2008 Hironao Komatsu
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,6 +17,10 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,12 +32,11 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include "breeze.h"
-#include "config.h"
-
-#include "getopt.h"
+#include "commands.h"
+#include "mainwin.h"
 
 /*
  *  Edit here to grab another key.
@@ -52,6 +55,11 @@ static unsigned int NumLockMask, CapsLockMask, ScrollLockMask;
 static Display *dpy = NULL;
 static Window root;
 
+static void signal_handler(int sig);
+static void init_keyboard(void);
+static void grab_key(int keycode, unsigned int modifiers, Window w);
+static unsigned int get_modifiers_to_grab(const char *str);
+static void initialize(int argc, char **argv);
 static void finalize(void);
 
 static void signal_handler(int sig)
@@ -77,7 +85,7 @@ static void signal_handler(int sig)
  *  Functions init_keyboard and grab_key are originally from keylaunch.c;
  *  written by Ken Lynch and Stefan Pfetzing.
  */
-static void init_keyboard()
+static void init_keyboard(void)
 {
   XModifierKeymap *xmk = NULL;
   KeyCode *map;
@@ -118,23 +126,6 @@ static void grab_key(int keycode, unsigned int modifiers, Window w)
   }
 }
 
-static void help(void)
-{
-  printf("Breeze " VERSION "\n"
-	 "  An application launcher with command-line style\n"
-	 "\n"
-	 "Usage: breeze [options...]\n"
-	 "\n"
-	 "Options:\n"
-	 "  -k, --key=KEY      sets KEY as the key to grab\n"
-	 "  -m, --modifier=MOD sets MOD as the modifier of KEY\n"
-	 "  -h, --help         show this message\n"
-	 "  -v, --version      same as --help\n"
-	 "\n"
-	 "MOD is one of:\n"
-	 "  Control, Shift, Mod1, Mod2, Mod3, Mod4, Mod5\n");
-}
-
 static unsigned int get_modifiers_to_grab(const char *str)
 {
   const char *modifiers_str[] = {
@@ -148,41 +139,58 @@ static unsigned int get_modifiers_to_grab(const char *str)
     if (strcmp(str, modifiers_str[i]) == 0)
       return modifiers[i];
 
-  fprintf(stderr, "breeze: warning: %s is not a valid modifier.\n", str);
-
   return 0;
 }
 
 static void initialize(int argc, char **argv)
 {
-  int opt;
-  const struct option opts[] = {
-    { "key", required_argument, NULL, 'k' },
-    { "modifier", required_argument, NULL, 'm' },
-    { "help", no_argument, NULL, 'h' },
-    { "version", no_argument, NULL, 'v' },
-    { 0, 0, 0, 0 }
+  gchar *keysym_str = NULL;
+  gchar **modifier_str_v = NULL;
+
+  GOptionContext *context;
+  const GOptionEntry entries[] = {
+    { "key", 'k', 0, G_OPTION_ARG_STRING, &keysym_str,
+      _("set KEY as the key to grab"), "KEY" },
+    { "modifier", 'm', 0, G_OPTION_ARG_STRING_ARRAY, &modifier_str_v,
+      _("set MOD as the modifier of KEY"), "MOD" },
+    { NULL },
   };
+  const gchar *desc = _("MOD is one of:\n"
+			"  Control, Shift, Mod1, Mod2, Mod3, Mod4, Mod5\n");
+  GError *err = NULL;
+
   KeySym keysym = 0L;
   unsigned int modifiers = 0;
   struct sigaction sa;
 
-  while ((opt = getopt_long(argc, argv, "k:m:hv", opts, NULL)) != -1) {
-    switch (opt) {
-    case 'k':
-      keysym = XStringToKeysym(optarg);
-      if (keysym == NoSymbol)
-	fprintf(stderr, "breeze: warning: %s is not a valid key.\n", optarg);
-      break;
-    case 'm':
-      modifiers |= get_modifiers_to_grab(optarg);
-      break;
-    case 'h':
-    case 'v':
-      help();
-      exit(0);
-    case '?':
-      exit(0);
+  context = g_option_context_new("breeze");
+  g_option_context_add_main_entries(context, entries, NULL);
+  g_option_context_add_group(context, gtk_get_option_group(TRUE));
+  g_option_context_set_help_enabled(context, TRUE);
+  g_option_context_set_description(context, desc);
+  g_option_context_parse(context, &argc, &argv, &err);
+
+  if (err) {
+    g_critical(_("Error handling option: %s\n"), err->message);
+    g_error_free(err);
+    exit(1);
+  }
+
+  if (keysym_str) {
+    keysym = XStringToKeysym(keysym_str);
+    if (keysym == NoSymbol)
+      g_warning(_("unknown key: %s. ignored.\n"), keysym_str);
+  }
+  if (modifier_str_v) {
+    gchar** p;
+
+    for (p = modifier_str_v; *p; p++) {
+      unsigned int mod = get_modifiers_to_grab(*p);
+
+      if (mod == 0)
+	g_warning(_("unknown modifier key: %s. ignored.\n"), *p);
+      else
+	modifiers |= mod;
     }
   }
 
@@ -196,12 +204,12 @@ static void initialize(int argc, char **argv)
   sigaction(SIGCHLD, &sa, NULL);
 
   if ((dpy = XOpenDisplay(NULL)) == NULL) {
-    fprintf(stderr, "Can't open display.\n");
+    g_critical(_("Can't open display.\n"));
     exit(1);
   }
 
   if (initialize_commands()) {
-    fprintf(stderr, "Can't initialize commands table.\n");
+    g_critical(_("Can't initialize commands table.\n"));
     XCloseDisplay(dpy);
     exit(1);
   }
@@ -235,17 +243,29 @@ static void finalize(void)
 
 int main(int argc, char **argv)
 {
+#ifdef ENABLE_NLS
+  bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+  textdomain(GETTEXT_PACKAGE);
+#endif
+
   initialize(argc, argv);
 
   while (continue_loop_flag) {
-    while (XPending(dpy)) {
+    if (XPending(dpy) > 0) {
       XEvent ev;
       XNextEvent(dpy, &ev);
 
       if (ev.type == KeyPress &&
 	  ev.xkey.keycode == key_to_grab &&
-	  ev.xkey.state & modifiers_to_grab)
-	popup_mainwin();
+	  ev.xkey.state & modifiers_to_grab) {
+	GtkWidget *win = main_win_new();
+	main_win_run(MAIN_WIN(win));
+      }
+    }
+    while (XPending(dpy) > 0) {
+      XEvent ev;
+      XNextEvent(dpy, &ev);
     }
     usleep(250);
   }
